package lc.esp.sdk;

import lc.mecha.json.JSONObject;

import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.TextMessage;
import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

public class ESPMessage {
    private JSONObject payload;
    private ESPAddress replyTo;
    private String correlationId;
    private final Map<String, Serializable> parameters = new HashMap<>();

    public ESPMessage(TextMessage msg) throws JMSException {
        payload = new JSONObject(msg.getText());
        correlationId = msg.getJMSCorrelationID();
        Destination dst = msg.getJMSReplyTo();
        if (dst != null) {
            replyTo = new ESPAddress(dst);
        } else {
            replyTo = null;
        }
    }

    public JSONObject getPayload() {
        return payload;
    }

    /**
     * Set a parameter in the message. This parameter may later be used to filter
     * messages during the subscription process (by creating a selector).
     * <p>
     * All parameters which begin with underscore are reserved by ESP.
     */
    public void setParameter(String name, Serializable value) {
        parameters.put(name, value);
    }

    public ESPMessage() {

    }

    public Serializable getParameter(String name) {
        return parameters.get(name);
    }

    public ESPMessage(JSONObject payload, ESPAddress replyTo, String correlationId) {
        this.payload = payload;
        this.replyTo = replyTo;
        this.correlationId = correlationId;
    }

    public void setPayload(JSONObject payload) {
        this.payload = payload;
    }

    public ESPMessage(JSONObject payload) {
        this(payload, null, null);
    }

    public String getCorrelationId() {
        return correlationId;
    }

    @Override
    public String toString() {
        return "ESPMessage{" +
                "payload=" + payload +
                ", replyTo=" + replyTo +
                ", correlationId='" + correlationId + '\'' +
                '}';
    }

    public ESPAddress getReplyTo() {
        return replyTo;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        ESPMessage that = (ESPMessage) o;
        return Objects.equals(payload, that.payload);
    }

    @Override
    public int hashCode() {
        return Objects.hash(payload);
    }

    public void setReplyTo(ESPAddress replyTo) {
        this.replyTo = replyTo;
    }

    public void setCorrelationId(String correlationId) {
        this.correlationId = correlationId;
    }
}
